"""Runserver command with livereload"""
from optparse import make_option
try:
    from urllib.request import urlopen
except ImportError:  # Python 2 fall back
    from urllib2 import urlopen

from django.conf import settings
from django.core.management.color import color_style


if 'django.contrib.staticfiles' in settings.INSTALLED_APPS:
    from django.contrib.staticfiles.management.commands.runserver import \
        Command as RunserverCommand
else:
    from django.core.management.commands.runserver import \
        Command as RunserverCommand


class Command(RunserverCommand):
    """
    Command for running the development server with LiveReload.
    """
    # Allow a settings option to set the default port
    default_port = getattr(settings, 'LIVERELOAD_PORT', '35729')
    # Allow a settings option to set the default host
    default_host = getattr(settings, 'LIVERELOAD_HOST', 'localhost')

    if callable(getattr(RunserverCommand, 'add_arguments', None)):
        def add_arguments(self, parser):
            super(Command, self).add_arguments(parser)
            parser.add_argument('--nolivereload', action='store_false',
                                dest='use_livereload', default=True,
                                help='Tells Django to NOT use LiveReload.')
            parser.add_argument('--livereload-port', action='store',
                                dest='livereload_port',
                                default=self.default_port,
                                help='Port where LiveReload listen.')
            parser.add_argument('--livereload-host', action='store',
                                dest='livereload_host',
                                default=self.default_host,
                                help='Host where LiveReload is running.')

    else:
        option_list = RunserverCommand.option_list + (
            make_option('--nolivereload', action='store_false',
                        dest='use_livereload', default=True,
                        help='Tells Django to NOT use LiveReload.'),
            make_option('--livereload-port', action='store',
                        dest='livereload_port', default=default_port,
                        help='Port where LiveReload listen.'),
            make_option('--livereload-host', action='store',
                        dest='livereload_host', default=default_host,
                        help='Host where LiveReload is running.'),
        )

    help = 'Starts a lightweight Web server for development with LiveReload.'

    def message(self, message, verbosity=1, style=None):
        if verbosity:
            if style:
                message = style(message)
            self.stdout.write(message)

    def livereload_request(self, **options):
        """
        Performs the LiveReload request.
        """
        style = color_style()
        verbosity = int(options['verbosity'])
        host = '%s:%s' % (options['livereload_host'],
                          options['livereload_port'])
        try:
            urlopen('http://%s/changed?files=.' % host)
            self.message('LiveReload request emitted.\n',
                         verbosity, style.HTTP_INFO)
        except IOError as e:
            self.stdout.write(
                self.style.WARNING('LiveReload exception: %s' % e)
            )
            pass

    def get_handler(self, *args, **options):
        """
        Entry point to plug the LiveReload feature.
        """
        handler = super(Command, self).get_handler(*args, **options)
        if options['use_livereload']:
            self.livereload_request(**options)
        return handler



"""
Middleware for injecting the live-reload script.
"""
from bs4 import BeautifulSoup

from django.conf import settings
from django.utils.encoding import smart_str

try:
    import html5lib  # noqa
    html5lib_is_installed = True
except ImportError:
    html5lib_is_installed = False


IS_HTML5 = getattr(settings, 'LIVERELOAD_HTML5', html5lib_is_installed)

PARSER = 'html.parser'
if IS_HTML5:
    PARSER = 'html5lib'


class LiveReloadScript(object):
    """
    Inject the live-reload script into your webpages.
    """

    def process_response(self, request, response):
        if response.status_code != 200:
            return response

        content_type = response.get(
            'Content-Type', '').split(';')[0].strip().lower()
        if content_type not in ['text/html', 'application/xhtml+xml']:
            return response

        soup = BeautifulSoup(
            smart_str(response.content), PARSER)

        if not getattr(soup, 'head', None):
            return response

        script = soup.new_tag(
            'script', src='http://localhost:35729/livereload.js')
        soup.head.append(script)

        response.content = str(soup)

        return response



"""Setup script of django-livereload"""
from setuptools import setup
from setuptools import find_packages

import livereload

setup(
    name='django-livereload',
    version=livereload.__version__,

    description='LiveReload with the Django development server',
    long_description=open('README.rst').read(),

    keywords='django, server, runserver, livereload',

    author=livereload.__author__,
    author_email=livereload.__email__,
    url=livereload.__url__,

    packages=find_packages(),
    classifiers=[
        'Framework :: Django',
        'Environment :: Web Environment',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development :: Libraries :: Python Modules'],

    license=livereload.__license__,
    include_package_data=True,
    install_requires=['beautifulsoup4>=4.3.2'],
)
